@@ -6,20 +6,21 @@
 import logging
 import os
 import shutil
-import sys
 import tempfile
+import warnings
+
+import pytest
 
 import zmq.auth
-from zmq.auth.ioloop import IOLoopAuthenticator
 from zmq.auth.thread import ThreadAuthenticator
 
-from zmq.eventloop import ioloop, zmqstream
-from zmq.tests import (BaseZMQTestCase, SkipTest)
+from zmq.utils.strtypes import u
+from zmq.tests import BaseZMQTestCase, SkipTest, skip_pypy
 
 
 class BaseAuthTestCase(BaseZMQTestCase):
     def setUp(self):
-        if zmq.zmq_version_info() < (4,0):
+        if zmq.zmq_version_info() < (4, 0):
             raise SkipTest("security is new in libzmq 4.0")
         try:
             zmq.curve_keypair()
@@ -31,17 +32,17 @@ class BaseAuthTestCase(BaseZMQTestCase):
         self.auth = self.make_auth()
         self.auth.start()
         self.base_dir, self.public_keys_dir, self.secret_keys_dir = self.create_certs()
-    
+
     def make_auth(self):
         raise NotImplementedError()
-    
+
     def tearDown(self):
         if self.auth:
             self.auth.stop()
             self.auth = None
         self.remove_certs(self.base_dir)
         super(BaseAuthTestCase, self).tearDown()
-    
+
     def create_certs(self):
         """Create CURVE certificates for a test"""
 
@@ -58,18 +59,24 @@ class BaseAuthTestCase(BaseZMQTestCase):
         os.mkdir(public_keys_dir)
         os.mkdir(secret_keys_dir)
 
-        server_public_file, server_secret_file = zmq.auth.create_certificates(keys_dir, "server")
-        client_public_file, client_secret_file = zmq.auth.create_certificates(keys_dir, "client")
+        server_public_file, server_secret_file = zmq.auth.create_certificates(
+            keys_dir, "server"
+        )
+        client_public_file, client_secret_file = zmq.auth.create_certificates(
+            keys_dir, "client"
+        )
 
         for key_file in os.listdir(keys_dir):
             if key_file.endswith(".key"):
-                shutil.move(os.path.join(keys_dir, key_file),
-                            os.path.join(public_keys_dir, '.'))
+                shutil.move(
+                    os.path.join(keys_dir, key_file), os.path.join(public_keys_dir, '.')
+                )
 
         for key_file in os.listdir(keys_dir):
             if key_file.endswith(".key_secret"):
-                shutil.move(os.path.join(keys_dir, key_file),
-                            os.path.join(secret_keys_dir, '.'))
+                shutil.move(
+                    os.path.join(keys_dir, key_file), os.path.join(secret_keys_dir, '.')
+                )
 
         return (base_dir, public_keys_dir, secret_keys_dir)
 
@@ -101,12 +108,27 @@ class TestThreadAuthentication(BaseAuthTestCase):
         port = server.bind_to_random_port(iface)
         client.connect("%s:%i" % (iface, port))
         msg = [b"Hello World"]
+        # run poll on server twice
+        # to flush spurious events
+        server.poll(100, zmq.POLLOUT)
+
         if server.poll(1000, zmq.POLLOUT):
-            server.send_multipart(msg)
+            try:
+                server.send_multipart(msg, zmq.NOBLOCK)
+            except zmq.Again:
+                warnings.warn("server set POLLOUT, but cannot send", RuntimeWarning)
+                return False
+        else:
+            return False
+
         if client.poll(1000):
-            rcvd_msg = client.recv_multipart()
-            self.assertEqual(rcvd_msg, msg)
-            result = True
+            try:
+                rcvd_msg = client.recv_multipart(zmq.NOBLOCK)
+            except zmq.Again:
+                warnings.warn("client set POLLIN, but cannot recv", RuntimeWarning)
+            else:
+                self.assertEqual(rcvd_msg, msg)
+                result = True
         return result
 
     def test_null(self):
@@ -117,7 +139,7 @@ class TestThreadAuthentication(BaseAuthTestCase):
         self.auth = None
         # use a new context, so ZAP isn't inherited
         self.context = self.Context()
-        
+
         server = self.socket(zmq.PUSH)
         client = self.socket(zmq.PULL)
         self.assertTrue(self.can_connect(server, client))
@@ -196,7 +218,7 @@ class TestThreadAuthentication(BaseAuthTestCase):
         certs = self.load_certs(self.secret_keys_dir)
         server_public, server_secret, client_public, client_secret = certs
 
-        #Try CURVE authentication - without configuring server, connection should fail
+        # Try CURVE authentication - without configuring server, connection should fail
         server = self.socket(zmq.PUSH)
         server.curve_publickey = server_public
         server.curve_secretkey = server_secret
@@ -207,7 +229,7 @@ class TestThreadAuthentication(BaseAuthTestCase):
         client.curve_serverkey = server_public
         self.assertFalse(self.can_connect(server, client))
 
-        #Try CURVE authentication - with server configured to CURVE_ALLOW_ANY, connection should pass
+        # Try CURVE authentication - with server configured to CURVE_ALLOW_ANY, connection should pass
         self.auth.configure_curve(domain='*', location=zmq.auth.CURVE_ALLOW_ANY)
         server = self.socket(zmq.PUSH)
         server.curve_publickey = server_public
@@ -221,15 +243,15 @@ class TestThreadAuthentication(BaseAuthTestCase):
 
         # Try CURVE authentication - with server configured, connection should pass
         self.auth.configure_curve(domain='*', location=self.public_keys_dir)
-        server = self.socket(zmq.PUSH)
+        server = self.socket(zmq.PULL)
         server.curve_publickey = server_public
         server.curve_secretkey = server_secret
         server.curve_server = True
-        client = self.socket(zmq.PULL)
+        client = self.socket(zmq.PUSH)
         client.curve_publickey = client_public
         client.curve_secretkey = client_secret
         client.curve_serverkey = server_public
-        self.assertTrue(self.can_connect(server, client))
+        assert self.can_connect(client, server)
 
         # Remove authenticator and check that a normal connection works
         self.auth.stop()
@@ -240,9 +262,123 @@ class TestThreadAuthentication(BaseAuthTestCase):
         client = self.socket(zmq.PULL)
         self.assertTrue(self.can_connect(server, client))
 
+    def test_curve_callback(self):
+        """threaded auth - CURVE with callback authentication"""
+        self.auth.allow('127.0.0.1')
+        certs = self.load_certs(self.secret_keys_dir)
+        server_public, server_secret, client_public, client_secret = certs
+
+        # Try CURVE authentication - without configuring server, connection should fail
+        server = self.socket(zmq.PUSH)
+        server.curve_publickey = server_public
+        server.curve_secretkey = server_secret
+        server.curve_server = True
+        client = self.socket(zmq.PULL)
+        client.curve_publickey = client_public
+        client.curve_secretkey = client_secret
+        client.curve_serverkey = server_public
+        self.assertFalse(self.can_connect(server, client))
+
+        # Try CURVE authentication - with callback authentication configured, connection should pass
+
+        class CredentialsProvider(object):
+            def __init__(self):
+                self.client = client_public
+
+            def callback(self, domain, key):
+                if key == self.client:
+                    return True
+                else:
+                    return False
+
+        provider = CredentialsProvider()
+        self.auth.configure_curve_callback(credentials_provider=provider)
+        server = self.socket(zmq.PUSH)
+        server.curve_publickey = server_public
+        server.curve_secretkey = server_secret
+        server.curve_server = True
+        client = self.socket(zmq.PULL)
+        client.curve_publickey = client_public
+        client.curve_secretkey = client_secret
+        client.curve_serverkey = server_public
+        self.assertTrue(self.can_connect(server, client))
+
+        # Try CURVE authentication - with callback authentication configured with wrong key, connection should not pass
+
+        class WrongCredentialsProvider(object):
+            def __init__(self):
+                self.client = "WrongCredentials"
+
+            def callback(self, domain, key):
+                if key == self.client:
+                    return True
+                else:
+                    return False
+
+        provider = WrongCredentialsProvider()
+        self.auth.configure_curve_callback(credentials_provider=provider)
+        server = self.socket(zmq.PUSH)
+        server.curve_publickey = server_public
+        server.curve_secretkey = server_secret
+        server.curve_server = True
+        client = self.socket(zmq.PULL)
+        client.curve_publickey = client_public
+        client.curve_secretkey = client_secret
+        client.curve_serverkey = server_public
+        self.assertFalse(self.can_connect(server, client))
+
+    @skip_pypy
+    def test_curve_user_id(self):
+        """threaded auth - CURVE"""
+        self.auth.allow('127.0.0.1')
+        certs = self.load_certs(self.secret_keys_dir)
+        server_public, server_secret, client_public, client_secret = certs
+
+        self.auth.configure_curve(domain='*', location=self.public_keys_dir)
+        server = self.socket(zmq.PULL)
+        server.curve_publickey = server_public
+        server.curve_secretkey = server_secret
+        server.curve_server = True
+        client = self.socket(zmq.PUSH)
+        client.curve_publickey = client_public
+        client.curve_secretkey = client_secret
+        client.curve_serverkey = server_public
+        assert self.can_connect(client, server)
+
+        # test default user-id map
+        client.send(b'test')
+        msg = self.recv(server, copy=False)
+        assert msg.bytes == b'test'
+        try:
+            user_id = msg.get('User-Id')
+        except zmq.ZMQVersionError:
+            pass
+        else:
+            assert user_id == u(client_public)
+
+        # test custom user-id map
+        self.auth.curve_user_id = lambda client_key: u'custom'
+
+        client2 = self.socket(zmq.PUSH)
+        client2.curve_publickey = client_public
+        client2.curve_secretkey = client_secret
+        client2.curve_serverkey = server_public
+        assert self.can_connect(client2, server)
+
+        client2.send(b'test2')
+        msg = self.recv(server, copy=False)
+        assert msg.bytes == b'test2'
+        try:
+            user_id = msg.get('User-Id')
+        except zmq.ZMQVersionError:
+            pass
+        else:
+            assert user_id == u'custom'
+
 
 def with_ioloop(method, expect_success=True):
     """decorator for running tests with an IOLoop"""
+
     def test_method(self):
         r = method(self)
 
@@ -251,32 +387,42 @@ def with_ioloop(method, expect_success=True):
             self.pullstream.on_recv(self.on_message_succeed)
         else:
             self.pullstream.on_recv(self.on_message_fail)
-        
+
         loop.call_later(1, self.attempt_connection)
         loop.call_later(1.2, self.send_msg)
-        
+
         if expect_success:
             loop.call_later(2, self.on_test_timeout_fail)
         else:
             loop.call_later(2, self.on_test_timeout_succeed)
-        
+
         loop.start()
         if self.fail_msg:
             self.fail(self.fail_msg)
-        
+
         return r
+
     return test_method
 
+
 def should_auth(method):
     return with_ioloop(method, True)
 
+
 def should_not_auth(method):
     return with_ioloop(method, False)
 
+
 class TestIOLoopAuthentication(BaseAuthTestCase):
     """Test authentication running in ioloop"""
 
     def setUp(self):
+        try:
+            from tornado import ioloop
+        except ImportError:
+            pytest.skip("Requires tornado")
+        from zmq.eventloop import zmqstream
+
         self.fail_msg = None
         self.io_loop = ioloop.IOLoop()
         super(TestIOLoopAuthentication, self).setUp()
@@ -284,8 +430,10 @@ class TestIOLoopAuthentication(BaseAuthTestCase):
         self.client = self.socket(zmq.PULL)
         self.pushstream = zmqstream.ZMQStream(self.server, self.io_loop)
         self.pullstream = zmqstream.ZMQStream(self.client, self.io_loop)
-    
+
     def make_auth(self):
+        from zmq.auth.ioloop import IOLoopAuthenticator
+
         return IOLoopAuthenticator(self.context, io_loop=self.io_loop)
 
     def tearDown(self):
@@ -305,7 +453,7 @@ class TestIOLoopAuthentication(BaseAuthTestCase):
         """Send a message from server to a client"""
         msg = [b"Hello World"]
         self.pushstream.send_multipart(msg)
-    
+
     def on_message_succeed(self, frames):
         """A message was received, as expected."""
         if frames != [b"Hello World"]:
